@@ -63,7 +63,7 @@ maxNR <- function(fn, grad=NULL, hess=NULL, start,
    else {
       mControl <- control
    }
-   mControl <- maxControl(mControl, ...)
+   mControl <- addControlList(mControl, list(...), check=FALSE)
    ##
    argNames <- c(c("fn", "grad", "hess", "start",
                    "activePar", "fixed", "control"),
