@@ -1,33 +1,40 @@
 maxNRCompute <- function(fn,
-                         start, print.level=0,
-                  tol=1e-8, reltol=sqrt(.Machine$double.eps),
-                  gradtol=1e-6, steptol=1e-10,
-                  lambdatol=1e-6,
-                  qrtol=1e-10,
-                  iterlim=150,
+                         start, 
+                           # maximum lambda for Marquardt (1963)
                          finalHessian=TRUE,
                   bhhhHessian = FALSE,
                   fixed=NULL,
+                         control=maxControl(),
                   ...) {
    ## Newton-Raphson maximisation
    ## Parameters:
-   ## fn          - the function to be minimized.  Returns either scalar or
+   ## fn          - the function to be maximized.  Returns either scalar or
    ##               vector value with possible attributes 
    ##               constPar and newVal
    ##               fn must return the value with attributes 'gradient'
    ##               and 'hessian'
    ##               fn must have an argument sumObs
    ## start       - initial parameter vector (eventually w/names)
-   ## steptol     - minimum step size
-   ## lambdatol   - max lowest eigenvalue when forcing pos. definite H
-   ## qrtol       - tolerance for qr decomposition
-   ## ...         - extra arguments for fn()
-   ## The stopping criteria
-   ## tol         - maximum allowed absolute difference between sequential values
-   ## reltol      - maximum allowed reltive difference (stops if < reltol*(abs(fn) + reltol)
-   ## gradtol     - maximum allowed norm of gradient vector
+   ## control       MaxControl object:
+   ##     steptol     - minimum step size
+   ##     lambda0       initial Hessian corrector (see Marquardt, 1963, p 438)
+   ##     lambdaStep    how much Hessian corrector lambda is changed between
+   ##                   two lambda trials
+   ##                  (nu in Marquardt (1963, p 438)
+   ##     maxLambda     largest possible lambda (if exceeded will give step error)
+   ##     lambdatol   - max lowest eigenvalue when forcing pos. definite H
+   ##     qrtol       - tolerance for qr decomposition
+   ##     qac           How to handle the case where new function value is
+   ##               smaller than the original one:
+   ##                  "stephalving"   smaller step in the same direction
+   ##                  "marquardt"     Marquardt (1963) approach
+   ##     The stopping criteria
+   ##     tol         - maximum allowed absolute difference between sequential values
+   ##     reltol      - maximum allowed reltive difference (stops if < reltol*(abs(fn) + reltol)
+   ##     gradtol     - maximum allowed norm of gradient vector
    ## 
-   ## iterlim     - maximum # of iterations
+   ##     iterlim     - maximum # of iterations
+   ##     
    ## finalHessian  include final Hessian?  As computing final hessian does not carry any extra penalty for NR method, this option is
    ##               mostly for compatibility reasons with other maxXXX functions.
    ##               TRUE/something else  include
@@ -49,6 +56,9 @@ maxNRCompute <- function(fn,
    ##             2 - successive values within tolerance limit
    ##             3 - could not find a higher point (step error)
    ##             4 - iteration limit exceeded
+   ##             5 - infinite function value
+   ##             6  infinite gradient
+   ##             7  infinite Hessian
    ##             100 - initial value out of range
    ## message     character message describing the code
    ## last.step   only present if code == 3 (step error).  A list with following components:
@@ -59,20 +69,32 @@ maxNRCompute <- function(fn,
    ## fixed       logical vector, which parameters were treated as constant (fixed, inactive, non-free)
    ## iterations  number of iterations
    ## type        "Newton-Raphson maximisation"
-   
+   ##
+   ## References:
+   ## Marquardt (1963), "An algorithm for least-squares estimation of nonlinear
+   ##      parameters", J. Soc. Indust. Appl. Math 11(2), 431-441
+   ##      
    max.eigen <- function( M) {
       ## return maximal eigenvalue of (symmetric) matrix
       val <- eigen(M, symmetric=TRUE, only.values=TRUE)$values
       val[1]
       ## L - eigenvalues in decreasing order, [1] - biggest in abs value
    }
-
    ## -------------------------------------------------
-   maxim.type <- "Newton-Raphson maximisation"
+   if(slot(control, "qac") == "marquardt")
+      marquardt <- TRUE
+   else
+      marquardt <- FALSE
+   ##
+   maximType <- "Newton-Raphson maximisation"
+   if(marquardt) {
+      maximType <- paste(maximType, "with Marquardt (1963) Hessian correction")
+   }
    nimed <- names(start)
    nParam <- length(start)
-
    samm <- NULL
+                           # data for the last step that could not find a better
+                           # value
    I <- diag(rep(1, nParam))
                            # I is unit matrix
    start1 <- start
@@ -80,13 +102,13 @@ maxNRCompute <- function(fn,
    returnHessian <- ifelse( bhhhHessian, "BHHH", TRUE )
    f1 <- fn(start1, fixed = fixed, sumObs = TRUE,
       returnHessian = returnHessian, ...)
-   if(print.level > 2) {
+   if(slot(control, "printLevel") > 2) {
       cat("Initial function value:", f1, "\n")
    }
    if(any(is.na( f1))) {
       result <- list(code=100, message=maximMessage("100"),
                      iterations=0,
-                     type=maxim.type)
+                     type=maximType)
       class(result) <- "maxim"
       return(result)
    }
@@ -94,7 +116,7 @@ maxNRCompute <- function(fn,
                                         # we stop at +Inf but not at -Inf
       result <- list(code=5, message=maximMessage("5"),
                      iterations=0,
-                     type=maxim.type)
+                     type=maximType)
       class(result) <- "maxim"
       return(result)
    }
@@ -107,7 +129,7 @@ maxNRCompute <- function(fn,
          " as argument 'hess': ignoring argument 'hess'" )
    }
    G1 <- attr( f1, "gradient" )
-   if(print.level > 2) {
+   if(slot(control, "printLevel") > 2) {
       cat("Initial gradient value:\n")
       print(G1)
    }
@@ -122,7 +144,7 @@ maxNRCompute <- function(fn,
          ") not equal to the no. of parameters (", nParam, ")" )
    }
    H1 <- attr( f1, "hessian" )
-   if(print.level > 3) {
+   if(slot(control, "printLevel") > 3) {
       cat("Initial Hessian value:\n")
       print(H1)
    }
@@ -130,7 +152,7 @@ maxNRCompute <- function(fn,
                            # Allow the user program to return a
                            # single NA in case of out of support or
                            # other problems
-      if(any(is.na(H1)))
+      if(is.na(H1))
           stop("NA in the initial Hessian")
    }
    if(any(is.na(H1[!fixed, !fixed]))) {
@@ -139,7 +161,7 @@ maxNRCompute <- function(fn,
    if(any(is.infinite(H1))) {
       stop("Infinite initial Hessian")
    }
-   if( print.level > 1) {
+   if( slot(control, "printLevel") > 1) {
       cat( "----- Initial parameters: -----\n")
       cat( "fcn value:",
       as.vector(f1), "\n")
@@ -149,16 +171,23 @@ maxNRCompute <- function(fn,
       print(a)
       cat( "Condition number of the (active) hessian:",
           kappa( H1[!fixed, !fixed]), "\n")
-      if( print.level > 3) {
+      if( slot(control, "printLevel") > 3) {
          print( H1)
       }
    }
+   lambda1 <- slot(control, "marquardt_lambda0")
+   step <- 1
+   ## ---------------- Main interation loop ------------------------
    repeat {
-      if( iter >= iterlim) {
+      lambda <- lambda1
+      if( iter >= slot(control, "iterlim")) {
          code <- 4; break
       }
       iter <- iter + 1
-      lambda <- 0
+      if(!marquardt) {
+         lambda <- 0
+                           # assume the function is concave at start0
+      }
       start0 <- start1
       f0 <- f1
       G0 <- G1
@@ -169,23 +198,37 @@ maxNRCompute <- function(fn,
       if(any(is.na(H0[!fixed, !fixed]))) {
          stop("NA in Hessian (at the iteration start)")
       }
-      step <- 1
-      H <- H0
+      if(marquardt) {
+         lambda1 <- lambda/slot(control, "marquardt_lambdaStep")
+                           # initially we try smaller lambda
+                           # lambda1: current lambda for calculations
+         H <- H0 - lambda1*I
+      }
+      else {
+         step <- 1
+         H <- H0
+      }
       ## check whether hessian is positive definite
-      while((me <- max.eigen( H[!fixed,!fixed,drop=FALSE])) >= -lambdatol |
-         (qRank <- qr(H[!fixed,!fixed], tol=qrtol)$rank) < sum(!fixed)) {
-                                        # maximum eigenvalue -> negative definite
-                                        # qr()$rank -> singularity
-         lambda <- abs(me) + lambdatol + min(abs(diag(H)[!fixed]))/1e7
+      if((me <- max.eigen( H[!fixed,!fixed,drop=FALSE])) >= -slot(control, "lambdatol") |
+                (qRank <- qr(H[!fixed,!fixed], tol=slot(control, "qrtol"))$rank) < sum(!fixed)) {
+                           # maximum eigenvalue -> negative definite
+                           # qr()$rank -> singularity
+         lambda1 <- abs(me) + slot(control, "lambdatol") + min(abs(diag(H)[!fixed]))/1e7
                            # The third term corrects numeric singularity.  If diag(H) only contains large values,
                            # (H - (a small number)*I) == H because of finite precision
-         H <- H - lambda*I
-                                        # how to make it better?
+         H <- (H0 - lambda1*I)
+                           # could we multiply it with something like (for stephalving)
+                           #     *abs(me)*lambdatol
+                           # -lambda*I makes the Hessian (barely)
+                           # negative definite.
+                           # *me*lambdatol keeps the scale roughly
+                           # the same as it was before -lambda*I
       }
       amount <- vector("numeric", nParam)
       amount[!fixed] <- qr.solve(H[!fixed,!fixed,drop=FALSE],
-                                    G0[!fixed], tol=qrtol)
+                                    G0[!fixed], tol=slot(control, "qrtol"))
       start1 <- start0 - step*amount
+                           # note: step is always 1 for Marquardt method
       f1 <- fn(start1, fixed = fixed, sumObs = TRUE,
          returnHessian = returnHessian, ...)
                            # The call calculates new function,
@@ -202,22 +245,41 @@ maxNRCompute <- function(fn,
       ## Are we asked to write in a new value for some of the parameters?
       if(is.null(newVal <- attr(f1, "newVal"))) {
          ## no ...
-         while( any(is.na(f1)) || ( ( sum(f1) < sum(f0) ) && ( step >= steptol))) {
+         if(marquardt) {
+            stepOK <- lambda1 <= slot(control, "marquardt_maxLambda")
+         }
+         else {
+            stepOK <- step >= slot(control, "steptol")
+         }
+         while( any(is.na(f1)) || ( ( sum(f1) < sum(f0) ) & stepOK)) {
                                         # We end up in a NA or a higher value.
                                         # try smaller step
-            step <- step/2
+            if(marquardt) {
+               lambda1 <- lambda1*slot(control, "marquardt_lambdaStep")
+               H <- (H0 - lambda1*I)
+               amount[!fixed] <- qr.solve(H[!fixed,!fixed,drop=FALSE],
+                                          G0[!fixed], tol=slot(control, "qrtol"))
+            }
+            else {
+               step <- step/2
+            }
             start1 <- start0 - step*amount
-            if(print.level > 2) {
-               if(print.level > 3) {
+            if(slot(control, "printLevel") > 2) {
+               if(slot(control, "printLevel") > 3) {
                   cat("Try new parameters:\n")
                   print(start1)
                }
-               cat("function value difference", f1 - f0, "-> step", step,
-                   "\n")
+               cat("function value difference", f1 - f0)
+               if(marquardt) {
+                  cat(" -> lambda", lambda1, "\n")
+               }
+               else {
+                  cat(" -> step", step, "\n")
+               }
             }
             f1 <- fn(start1, fixed = fixed, sumObs = TRUE,
                returnHessian = returnHessian, ...)
-                           # WTF does the 'returnHessian' here ?
+                           # WTF does the 'returnHessian' do here ?
             ## Find out the constant parameters -- these may be other than
             ## with full step
             constPar <- attr(f1, "constPar")
@@ -235,7 +297,13 @@ maxNRCompute <- function(fn,
                }
             }
          }
-         if(step < steptol) {
+         if(marquardt) {
+            stepOK <- lambda1 <= slot(control, "marquardt_maxLambda")
+         }
+         else {
+            stepOK <- step >= slot(control, "steptol")
+         }
+         if(!stepOK) {
             # we did not find a better place to go...
             start1 <- start0
             f1 <- f0
@@ -246,7 +314,7 @@ maxNRCompute <- function(fn,
          ## Note, this may result in a lower function value,
          ## hence we do not check f1 > f0
          start1[newVal$index] <- newVal$val
-         if( print.level > 0 ) {
+         if( slot(control, "printLevel") > 0 ) {
             cat( "Keeping parameter(s) ",
                paste( newVal$index, collapse = ", " ),
                " at the fixed values ",
@@ -261,30 +329,27 @@ maxNRCompute <- function(fn,
          cat("Iteration", iter, "\n")
          cat("Parameter:\n")
          print(start1)
-         if(length(G1) < 30) {
-            cat("Gradient:\n")
-            print(G1)
-         }
+         print(head(G1, n=30))
          stop("NA in gradient")
       }
       if(any(is.infinite(G1))) {
          code <- 6; break;
       }
       H1 <- attr( f1, "hessian" )
-      if( print.level > 1) {
+      if( slot(control, "printLevel") > 1) {
         cat( "-----Iteration", iter, "-----\n")
       }
       if(any(is.infinite(H1))) {
          code <- 7; break
       }
-      if(print.level > 2) {
-         cat( "lambda ", lambda, " step", step, " fcn value:",
+      if(slot(control, "printLevel") > 2) {
+         cat( "lambda ", lambda1, " step", step, " fcn value:",
             formatC(as.vector(f1), digits=8, format="f"),  "\n")
          a <- cbind(amount, start1, G1, as.integer(!fixed))
          dimnames(a) <- list(names(start0), c("amount", "new param",
                                              "new gradient", "active"))
          print(a)
-         if( print.level > 3) {
+         if( slot(control, "printLevel") > 3) {
             cat("Hessian\n")
             print( H1)
          }
@@ -293,23 +358,29 @@ maxNRCompute <- function(fn,
                 kappa(H1[!fixed,!fixed,drop=FALSE]), "\n")
          }
       }
-      if( step < steptol) {
+      if( step < slot(control, "steptol")) {
+                           # wrong guess in step halving
          code <- 3; break
       }
-      if( sqrt( crossprod( G1[!fixed] ) ) < gradtol ) {
+      if(lambda1 > slot(control, "marquardt_maxLambda")) {
+                           # wrong guess in Marquardt method
+         code <- 3; break
+      }
+      if( sqrt( crossprod( G1[!fixed] ) ) < slot(control, "gradtol") ) {
          code <-1; break
       }
-      if(is.null(newVal) && sum(f1) - sum(f0) < tol) {
+      if(is.null(newVal) && sum(f1) - sum(f0) < slot(control, "tol")) {
          code <- 2; break
       }
-      if(is.null(newVal) && sum(f1) - sum(f0) < reltol * ( sum(f1) + reltol)) {
+      if(is.null(newVal) && abs(sum(f1) - sum(f0)) <
+         abs(slot(control, "reltol")*( sum(f1) + slot(control, "reltol")))) {
          code <- 2; break
       }
       if(any(is.infinite(f1)) && sum(f1) > 0) {
          code <- 5; break
       }
    }
-   if( print.level > 0) {
+   if( slot(control, "printLevel") > 0) {
       cat( "--------------\n")
       cat( maximMessage( code), "\n")
       cat( iter, " iterations\n")
@@ -354,22 +425,24 @@ maxNRCompute <- function(fn,
    attributes( f1 )$hessBoth <- NULL
    ##
    result <-list(
-                  maximum = unname( drop( f1 ) ),
+       maximum = unname( drop( f1 ) ),
                   estimate=start1,
                   gradient=drop(G1),
                  hessian=hessian,
                   code=code,
-                  message=maximMessage( code),
+       message=maximMessage( code),
                   last.step=samm,
-                                        # only when could not find a
-                                        # lower point
+                           # only when could not find a
+                           # lower point
                   fixed=fixed,
-                  iterations=iter,
-                  type=maxim.type)
+       iterations=iter,
+                  type=maximType)
    if( exists( "gradientObs" ) ) {
       result$gradientObs <- gradientObs
    }
-
+   result <- c(result, control=control)
+                           # attach the control parameters
+   ##
    class(result) <- c("maxim", class(result))
    invisible(result)
 }
